#ifndef _BALL_H
#define _BALL_H

#include "GameObject.h"
#include "Point.h"
#include "Direction.h"
#include <QColor>

class Ball : public GameObject {

	Point center;
	
	Point previousCenter;
	
	Point lastIterationPosition;
	
	Direction previousSpeed;

    float radius;

    float weight;

    QColor color;

    Direction speed;

	int number;		// ball number: 0..white, 1-7..solids, 8..black, 9-15..stripes

	bool moving;

  public:
	Ball();

	Ball(const Point & pcenter, float pradius, float pweight, const QColor & pcolor, int pnumber);

    // distance from another ball
	float distance(const Ball & ball) const;

    void move(float f);

    Point getCenter() const;

    Direction getSpeed() const;

    float getRadius() const;

    float getWeight() const;

	int getNumber() const;

	QColor getColor() const;

    void setSpeed(const Direction & newSpeed);

	void setCenter(const Point& p);

	Ball previousPosition() const;

    virtual void draw() const;

	virtual bool colliding(const Ball& b) const;	// is there a collision

	bool isWhite() const;
	
	bool isBlack() const;
	
	bool isSolid() const;
	
	bool isStripe() const;

	float computeOverlap(const Ball& b) const;

	void attenuate(float f);

	bool isMoving() const;

	bool wasStanding() const;

	// position before stroke - used to eliminate big energy losses when in touching position
	Point getLastIterationPosition() const;

	// ball out of bounds of the table
	bool isInPocket() const;

	bool operator==(const Ball& b) const;
};
#endif
