// represents a mathematical vector in 3D

#include "Direction.h"
#include "Point.h"

// constructor
Direction::Direction() : x(0), y(0), z(0) {}

// constructor
Direction::Direction(const Point & start, const Point & end) : x(end.getX()-start.getX()), y(end.getY()-start.getY()), z(end.getZ()-start.getZ()) {}

// constructor
Direction::Direction(float px, float py, float pz) : x(px), y(py), z(pz) {}

// copy constructor
Direction::Direction(const Direction& dir) : x(dir.getX()), y(dir.getY()), z(dir.getZ()) {}

// normalize - make the magnitude equal to zero
void Direction::normalize() {
	operator /= (size());
}

// returns a normalized version of the vector, but doesn't change this one
Direction Direction::normalized() const {
	Direction d(*this);
	d.operator/=(d.size());
	return d;
}


Direction Direction::operator-() const {
	return Direction(-x, -y, -z);
}

// get magnitude of the vector
float Direction::size() const {
	return sqrt(x*x + y*y + z*z);
}

Direction Direction::operator+(const Direction & dir) const {
	return Direction(x+dir.getX(), y+dir.getY(), z+dir.getZ());
}

Direction Direction::operator-(const Direction & dir) const {
	return operator+(-dir);
}

Direction Direction::operator*(float f) const {
	return Direction(f*x, f*y, f*z);
}

float Direction::operator*(const Direction& dir) const {
	return (x*dir.x + y*dir.y + z*dir.z);
}

Direction Direction::operator/(float f) const {
	return operator*(1/f);
}

// vector product
Direction Direction::operator%(const Direction & dir) const {
	return Direction(y*dir.getZ()-z*dir.getY(), z*dir.getX()-x*dir.getZ(), x*dir.getY()-y*dir.getX());
}

Direction& Direction::operator+=(const Direction & dir) {
	x += dir.getX();
	y += dir.getY();
	z += dir.getZ();
	return *this;
}

Direction& Direction::operator-=(const Direction & dir) {
	x -= dir.getX();
	y -= dir.getY();
	z -= dir.getZ();
	return *this;
}

Direction& Direction::operator*=(float f) {
	x *= f;
	y *= f;
	z *= f;
	return *this;
}

Direction& Direction::operator/=(float f) {
	return operator*=(1/f);
}

float Direction::getX() const {
	return x;
}

float Direction::getY() const {
	return y;
}

float Direction::getZ() const {
	return z;
}

// is the vector zero - magnitude == 0
bool Direction::isNull() const {
	return (x==0 && y==0 && z==0);
}

// equals operator
bool Direction::operator==(const Direction& d) const {
	return (x==d.x, y==d.y, z==d.z);
}