// represents a mathematical vector in 3D

#ifndef _DIRECTION_H
#define _DIRECTION_H

#include "globals.h"
#include <stdio.h>
#include <math.h>

class Point;

class Direction {
	float x;

    float y;

    float z;

  public:
	Direction();

    Direction(const Point & start, const Point & end);

    Direction(float px, float py, float pz);

	Direction(const Direction& dir);

    void normalize();

	Direction normalized() const;

    Direction operator-() const;

    // get magnitude of the vector
	float size() const;

	Direction operator+(const Direction & dir) const;

    Direction operator-(const Direction & dir) const;

    // dot product
	float operator*(const Direction& dir) const;

	Direction operator*(float f) const;
	
	Direction operator/(float f) const;

	// vector product
	Direction operator%(const Direction & dir) const;

    Direction& operator+=(const Direction & dir);

    Direction& operator-=(const Direction & dir);

    Direction& operator*=(float f);

	Direction& operator/=(float f);

    float getX() const;

    float getY() const;

    float getZ() const;

	bool isNull() const;

	bool operator==(const Direction& d) const;

};
#endif
