// brain of the application
// contains game rules
// controls ball moves
// looks for collisions and lets them be handled

#ifndef _GAME_H
#define _GAME_H


#include "Ball.h"
#include "Cuboid.h"
#include "GameObject.h"
#include "GameSetup.h"
#include "Cue.h"
#include "Collision.h"
#include "glwidget.h"

class Plane;
class GameSetup;
class Cue;

class Game {
  private:
	
	// game variables - contain information about state of the game
	int state;
	int activePlayer;

	bool resetCueBall;
	bool pocketedOther;
	bool pocketedMine;
	bool overheadView;

	vector<Ball> balls;		// balls present in the game

	vector<Cuboid> cuboids;		// boxes present in the game (including the table)

	GameSetup setup;	// game setup info

	Cue cue;

	bool moving;	// animation in progress - at least one ball is still moving

	void handleCollisions();	// find and solve potential collisions

	Ball* getWhite();

	void removeBall(int i);		// remove ball from game when it is pocketed

	void switchPlayers();

	void displayWin();		// display who won

  public:

	GLWidget* widget;
	
	Game(const string cfgFile);

	void newGame();		// reset game - reposition all balls, reset state variables

	void stroke();		// do a stroke - make the cue ball move

	void doIteration();	// one iteration of simulation

	void move(float t);	// move all balls

	void rotateCue(float x, float y);	// move cue to new position (upon mouse movement)

	void draw() const;

	void setStrength(int s);	// how much energy to put in the stroke

	void displayPlayer();	// display whose turn it is

	void changeView();

	void updateView();
};
#endif
