// represents a mathematical plane

#include "Plane.h"
#include "Ball.h"

Plane::Plane(const Point & ppoint, const Direction &dir1, const Direction &dir2) : point(ppoint), normal(dir1%dir2) {
	normal.normalize();
	d = -(getA()*point.getX() + getB()*point.getY() + getC()*point.getZ());
}

Plane::Plane(const Point & ppoint, const Direction & pnormal) : point(ppoint), normal(pnormal) {
	normal.normalize();
	d = -(getA()*point.getX() + getB()*point.getY() + getC()*point.getZ());
}

// constructor for null plane
Plane::Plane() : point(0,0,0), normal(0,0,0), d(0) {
}

Plane::Plane(const Plane& p) : d(p.getD()), point(p.getPoint()), normal(Direction(p.getA(), p.getB(), p.getC())) {
}

// d = (A*x + B*y + C*z + D) / sqrt(A^2 + B^2 + C^2)
float Plane::distance(const Ball & ball) const {
	Point c = ball.getCenter();
	return (getA()*c.getX() + getB()*c.getY() + getC()*c.getZ() + getD()) / sqrt(getA()*getA() + getB()*getB() + getC()*getC());
}

float Plane::getA() const {
	return normal.getX();
}

float Plane::getB() const {
	return normal.getY();
}

float Plane::getC() const {
	return normal.getZ();
}

float Plane::getD() const {
	return d;
}

Direction Plane::getNormal() const {
	return normal;
}

Point Plane::getPoint() const {
	return point;
}

bool Plane::isNull() const {
	return (getA()==0 && getB()==0 && getC()==0 && getD()==0);
}

// compute point of intersection of the plane and a direction vector
Point Plane::findIntersection(const Point& p, const Direction& d) const {
	// (A*p.x + B*p.y + C*p.z + D) / (A*d.x + B*d.y + C*d.z)
	float divider = getA()*d.getX() + getB()*d.getY() + getC()*d.getZ();
	if (divider == 0) {	// eliminate division by zero
		return p;
	}
	float t = -(getA()*p.getX() + getB()*p.getY() + getC()*p.getZ() + getD()) / (getA()*d.getX() + getB()*d.getY() + getC()*d.getZ());
	return Point(p.getX()+t*d.getX(), p.getY()+t*d.getY(), p.getZ()+t*d.getZ());
}