// global constants

#ifndef _GLOBALS_H
#define _GLOBALS_H

#include <assert.h>
#include <exception>
#include <string>

class Ball;

// exceptions
class insideCuboidException : public std::exception {
public:
	Ball* ball;
	insideCuboidException(Ball *b) : ball(b) {}
};

class fileErrorException : public std::exception {
public:
	std::string message;
	fileErrorException(const std::string m) : message(m) {}
};

class noWhiteBallException : public std::exception {
};

// constants
#define WIDTH 694
#define HEIGHT 559

#define VIEWER_POSITION_X 0
#define VIEWER_POSITION_Y 0
#define VIEWER_POSITION_Z 500

#define BACKGROUND_COLOR 10.f/255, 30.f/255, 30.f/255, 1

#define RADIUS 2.86
#define RADIUS_WHITE 2.86
#define WEIGHT 1.00
#define WEIGHT_WHITE 1.09
#define COLOR_SOLID QColor(50,0,0)
#define COLOR_STRIPE QColor(0,10,50)
#define COLOR_WHITE QColor(200,200,200)
#define COLOR_BLACK QColor(0,0,0)

#define DEFAULT_CFG_FILE "game_setup.cfg"
enum {READING_BALLS, READING_CUBOIDS};

#define TABLE_WIDTH	116.84
#define TABLE_LENGTH 233.68
#define TABLE_MARGIN 10
#define TABLE_WALL_HEIGHT 7
#define TABLE_POCKET 8.31		// width of the pocket is 11.75, so in each axis it makes 8.31
#define TABLE_POCKET_MIDDLE 13.02

#define CUE_COLOR 153,102,0
#define CUE_LENGTH 150

#define TIMER 20

#define ATTENUATION_GENERAL 0.998	// each iteration
#define ATTENUATION_BALLBALL 0.9	// ball-ball collision
#define ATTENUATION_BALLWALL 0.8	// ball-wall collision

#define MIN_SPEED 0.05	// this is equal to zero speed

#define BLACKBALL_X 80

#define WHITE_START_POSITION -TABLE_LENGTH/4, 0

#define MOUSE_SENSITIVITY 100


// game states
enum {NEWGAME, PLAYING_SOLID, PLAYING_STRIPE, WIN_SOLID, WIN_STRIPE, WIN, LOSE};
enum {PLAYING_FIRST=1, PLAYING_SECOND};

#endif