// openGL widget
// takes care of displaying (using a timer)
// and of monitoring user input (mouse movement and clicks)

#ifndef GLWIDGET_H
#define GLWIDGET_H

#include <QGLWidget>
#include <QtGui>
#include <QtOpenGL>
#include <qtimer.h>
#include "Point.h"
#include <string>

class Coolaaber;

//class representing the canvas that GL uses to draw
class GLWidget : public QGLWidget {
	Q_OBJECT

public:

	int strength;
	Point lastPosition;
	Coolaaber *cool;	// access to other parts of the application
	bool displayEnabled;

	GLWidget( QWidget *parent = 0 );
	QTimer *timer;

	void start(int interval);		// start the timer
	void stop();		// stop the timer
	
	void coolClass(Coolaaber* c);		// sets up access to other parts of the application

	void setLabel(const std::string& s);	// display message for the user

	void setView(const Point& viewer, const Point& center, const Direction& up);

protected:
    void initializeGL();
    void paintGL();
    void resizeGL(int width, int height);
	void mousePressEvent( QMouseEvent *event );
	void mouseMoveEvent( QMouseEvent *event );
	void mouseReleaseEvent( QMouseEvent *event );

public slots:
	void restartSlot();
	void strengthSlot();
	void strengthValueSlot(int s);
	void changeViewSlot();

protected slots:
	void timerSlot();

};

#endif
